'use strict'

var gulp            = require('gulp'),
    gutil           = require('gulp-util'),
    cleanDest       = require('gulp-clean-dest'),
    spritesmith     = require('gulp.spritesmith'),
    sass            = require('gulp-sass'),
    sourcemaps      = require('gulp-sourcemaps'),
    baseimg         = require('gulp-baseimg'),
    path            = require('path'),
    rebaseCssUrls   = require('gulp-rebase-css-urls'),
    concatCss       = require('gulp-concat-css'),
    cleanCSS        = require('gulp-clean-css'),
    runSequence     = require('run-sequence'),
    modernizr       = require('modernizr'),
    webpack         = require('webpack'),
    fs              = require('fs')

/*gulp.task('concatcss', function () {
    return gulp.src([
            'bower_components/bootstrap/dist/css/bootstrap.min.css',
            'bower_components/selectize/dist/css/selectize.bootstrap3.css',
            'bower_components/ion.rangeSlider/css/ion.rangeSlider.css',
            'bower_components/ion.rangeSlider/css/ion.rangeSlider.skinHTML5.css'
        ])
        .pipe(rebaseCssUrls('./css'))
        .pipe(concatCss('vendor.css', {
            inlineImports: true, // Inline any local import statement found
            rebaseUrls: false, // because it works incorrectly
            includePaths: [] // Include additional paths when inlining imports
        }))
        .pipe(cleanCSS({debug: true, compatibility: 'ie8', keepSpecialComments: 0}, function(details) {
            console.log(details.name + ': ' + details.stats.originalSize);
            console.log(details.name + ': ' + details.stats.minifiedSize);
        }))
        .pipe(gulp.dest('./css'));
});*/

/**
 * Build UI texture atlas
 */
gulp.task('sprite:ui', function () {
    return gulp.src('src/images/sprites/ui/*.png')
        .pipe(spritesmith({
            imgName: 'dist/img/ui.png',
            imgPath: '../img/ui.png',
            cssName: 'src/styles/sprites/_ui.scss',
            imgOpts: {quality: 75},
            cssVarMap: function (sprite) {
                sprite.name = 'sprite-' + sprite.name;
                //console.log(sprite)
            }
        }))
        .pipe(gulp.dest('.'))
})

/**
 * Build all texture atlases
 */
gulp.task('sprite', ['sprite:ui'])

/**
 * Build SASS -> CSS
 */
gulp.task('sass', function () {
    return gulp.src('src/styles/main.scss')
        .pipe(sass({outputStyle: 'compressed'}).on('error', sass.logError))
        .pipe(gulp.dest('dist/css'))
})

/**
 * Compiles SASS on file change
 */
gulp.task('sass:watch', function () {
    gulp.watch('src/styles/**/*.scss', ['sass'])
})

/**
 * Base64-encode raster images to data-uri in SASS partial
 */
gulp.task('base64:bitmaps', function() {
    var globs = ['*.jpeg', '*.jpg', '*.png', '*.gif']
    return gulp.src(globs.map(glob => path.join('src/images/base64', glob)))
        .pipe(baseimg({
            styleTemplate: path.join(__dirname, 'src/styles/base64/_bitmaps.scss.mustache'),
            styleName: '_bitmaps.scss'
        }))
        .pipe(gulp.dest('src/styles/base64'))
})

/**
 * Base64-encode SVG images to data-uri in SASS partial
 */
gulp.task('base64:svg', function() {
    var globs = ['*.svg']
    return gulp.src(globs.map(glob => path.join('src/images/base64', glob)))
        .pipe(baseimg({
            styleTemplate: path.join(__dirname, 'src/styles/base64/_svg.scss.mustache'),
            styleName: '_svg.scss'
        }))
        .pipe(gulp.dest('src/styles/base64'))
})

/**
 * Base64-encode all vector and raster images to data-uri in SASS partial
 */
gulp.task('base64', ['base64:bitmaps', 'base64:svg'])

/*gulp.task("modernizr", function(callback) {
    modernizr.build({
        "minify": false,
        "options": [
            "setClasses"
        ],
        "feature-detects": [
            'css/transforms', 'css/transforms3d', 'css/animations',
            'css/flexbox'
        ]
    }, function (result) {
        let wrappedResult = result + '\nmodule.exports = window.Modernizr;'
        fs.writeFile(__dirname + "/js-src/modernizr.js", wrappedResult, function(err) {
            if(err) {
                console.log(err);
            }
            callback();
        });
    });
});*/

gulp.task("webpack", function(callback) {
    let config = Object.create(require('./webpack.config'))
    webpack(config, function(err, stats) {
        if(err) throw new gutil.PluginError("webpack", err)
        gutil.log("[webpack]", stats.toString())
        callback()
    })
})

gulp.task("webpack:watch", function() {
    let config = Object.create(require('./webpack.config'))
    config.watch = true
    webpack(config, function(err, stats) {
        if(err) throw new gutil.PluginError("webpack", err)
        gutil.log("[webpack]", stats.toString())
    })
})

gulp.task('default', function(callback) {
    runSequence(
        ['sprite', 'base64'],
        'sass',
        'webpack',
        callback)
})